﻿using System;
using System.Numerics;
using System.Security.Cryptography;

namespace MillerRabinPrimZahlenTest
{
    class Program
    {
        static readonly int NumberLengthInByte = 128;
        static RandomNumberGenerator rng = RandomNumberGenerator.Create();

        static void Main(string[] args)
        {
            BigInteger number = GeneratePositivRandomNumber();

            bool isProbablePrime = false;
            while (!isProbablePrime)
            {
                isProbablePrime = IsProbablePrime(number, 150);
                Console.WriteLine("isProbablePrime {0,6}  getestet {1}", isProbablePrime, number.ToString());
                if (!isProbablePrime)
                {
                    number += BigInteger.One;
                }
                
            };

            Console.ReadLine();
        }

        static BigInteger GeneratePositivRandomNumber()
        {
            byte[] bytes = new byte[NumberLengthInByte];
            rng.GetBytes(bytes);

            BigInteger number = new BigInteger(bytes);
            return (number > BigInteger.Zero) ? number : -number;
        }

        // Miller-Rabin Test
        static bool IsProbablePrime(BigInteger number, int certainty)
        {
            if (number == 2 || number == 3)
                return true;
            if (number < 2 || number % 2 == 0)
                return false;

            BigInteger exponent = number - BigInteger.One;
            int nthRoot = 0;
            while (exponent % 2 == 0)
            {
                exponent /= 2;
                nthRoot++;
            }

            BigInteger randomBase, x;
            for (int i = 0; i < certainty; i++)
            {
                do
                {
                    randomBase = GeneratePositivRandomNumber();
                }
                while (randomBase < 2 || randomBase >= number - 2);

                x = BigInteger.ModPow(randomBase, exponent, number);
                if (x == BigInteger.One || x == number - BigInteger.One)
                    continue;

                for (int squared = 1; squared < nthRoot; squared++)
                {
                    // auf nicht triviale Wurzel testen
                    x = BigInteger.ModPow(x, 2, number);
                    if (x == BigInteger.One)
                        return false;
                    if (x == number - BigInteger.One)
                        break;
                }

                if (x != number - BigInteger.One)
                    return false;
            }
            return true;
        }
    }
}
